﻿"use strict";

function Game(controller) {
	this.controller = controller;
	this.simulation = new Simulation();
	controller.uiButton(200, 0, 340, "Buy 1 Ø", 16, function(game) {
		return function() {
			game.buy(1);
		};
	}(this));
	controller.uiButton(320, 0, 340, "Buy 10 Ø", 16, function(game) {
		return function() {
			game.buy(10);
		};
	}(this));
	controller.uiButton(450, 0, 340, "Buy 100 Ø", 16, function(game) {
		return function() {
			game.buy(100);
		};
	}(this));
	controller.uiButton(200, 0, 390, "Sell 1 Ø", 16, function(game) {
		return function() {
			game.sell(1);
		};
	}(this));
	controller.uiButton(320, 0, 390, "Sell 10 Ø", 16, function(game) {
		return function() {
			game.sell(10);
		};
	}(this));
	controller.uiButton(450, 0, 390, "Sell 100 Ø", 16, function(game) {
		return function() {
			game.sell(100);
		};
	}(this));

	controller.uiButton(830, 0, 340, "1x", 16, function(game) {
		return function() { game.speed(1); }
	}(this));
	controller.uiButton(900, 0, 340, "5x", 16, function(game) {
		return function() { game.speed(5); }
	}(this));
	controller.uiButton(970, 0, 340, "25x", 16, function(game) {
		return function() { game.speed(25); }
	}(this));
}



Game.prototype.start = function(controller) {
	for (var i = 0; i < 196; i++) {
		this.simulation.tick();
	}
	this.money = 50000;
	this.potatoes = 0;
	this.totalSpent = 0;
	this.totalGained = 0;
	controller.startTimer();
};



Game.prototype.speed = function(speed) {
	this.controller.timerMultiplier(speed);
};



Game.prototype.buy = function(amount) {
	var maxBuy = Math.floor(this.money / this.simulation.currentValue);
	if (amount > maxBuy)
		amount = maxBuy;
	if (amount === 0)
		return;
	var spent = Math.round(amount * this.simulation.currentValue);
	this.money -= spent;
	this.totalSpent += spent;
	this.potatoes += amount;
	this.simulation.recordBuy();
	$("#transactionList").prepend("<li>Buy " + amount + " at " + this.simulation.currentValueStr());
};



Game.prototype.sell = function(amount) {
	if (this.potatoes < amount)
		amount = this.potatoes;
	if (amount === 0)
		return;
	var gained = Math.round(amount * this.simulation.currentValue);
	this.money += gained;
	this.totalGained += gained;
	this.potatoes -= amount;
	this.simulation.recordSell();
	$("#transactionList").prepend("<li>Sell " + amount + " at " + this.simulation.currentValueStr());
};



Game.prototype.onTimer = function() {
	this.simulation.tick();
};



Game.prototype.render = function(render, xSize, ySize) {
	var context = render.context;

	// draw chart
	var chartXSize = xSize - 120;
	var values = this.simulation.getValues();
	context.save();
	context.translate(60.5, 15.5);

	// draw grid lines
	var power = Math.floor(Math.log((values.max - values.min) * 4) / Math.LN10) - 1;
	var power10 = Math.pow(10, power);
	var minGridY = Math.ceil(values.min / power10);
	var maxGridY = Math.floor(values.max / power10);
	context.font = "12px Verdana";
	for (var y = minGridY; y <= maxGridY; y++) {
		context.beginPath();
		var value = power10 * y;
		var gridLineY = Math.round(this.getY(value, values.min, values.max));
		context.moveTo(0, gridLineY);
		context.lineTo(chartXSize, gridLineY);
		context.strokeStyle = y % 10 === 0 ? "#9f9f9f" : "#bfbfbf";
		context.stroke();
		if (y % 10 === 0) {
			context.fillStyle = "#000000";
			context.fillText(value, -35, gridLineY + 3);
		}
	}

	// draw outline
	context.strokeStyle = "#000000";
	context.strokeRect(0, 0, chartXSize, 300);

	// draw chart values
	context.beginPath();
	context.lineCap = "square";
	context.moveTo(0, this.getY(values.values[0], values.min, values.max));
	for (var i = 1; i < values.values.length; i++) {
		context.lineTo(i * 5, this.getY(values.values[i], values.min, values.max));
	}
	context.strokeStyle = "#000000";
	context.stroke();
	context.fillStyle = "#000000";
	var currentY = this.getY(this.simulation.currentValue, values.min, values.max);
	context.fillText(this.simulation.currentValueStr(), chartXSize + 5, currentY + 3);

	// draw buys and sells
	for (var i = 0; i < values.buys.length; i++) {
		if (values.buys[i] !== null) {
			context.beginPath();
			context.arc(i * 5, this.getY(values.buys[i], values.min, values.max), 3,
					0, Math.PI * 2);
			context.fillStyle = "#B9BF13";
			context.fill();
		}
		if (values.sells[i] !== null) {
			context.beginPath();
			context.arc(i * 5, this.getY(values.sells[i], values.min, values.max), 3,
					0, Math.PI * 2);
			context.fillStyle = "#1812BC";
			context.fill();
		}
	}

	context.restore();



	// draw UI
	context.font = "18px Verdana";
	context.fillStyle = "#000000";
	render.drawText("$ " + this.money, 60, 350, 0);
	render.drawText("Ø " + this.potatoes, 60, 380, 0);
	render.drawText("Profit $" + (this.totalGained - this.totalSpent), 60, 440, 0);
	render.drawText("Total spent $" + this.totalSpent, 60, 465, 0);
	render.drawText("Total gained $" + this.totalGained, 60, 490, 0);

	var maxBuy = Math.floor(this.money / this.simulation.currentValue);
	render.drawText("Max buy: " + maxBuy, 570, 355, 0);
	var totalValue = this.money + Math.round(this.potatoes * this.simulation.currentValue);
	render.drawText("Total value: $" + totalValue, 570, 380, 0);
};



Game.prototype.getY = function(value, min, max) {
	return 300 - (value - min) / (max - min) * 300;
};